class GuosenConfig(object):
    host = '172.18.32.108'
    port = 3306
    user = 'root'
    passwd = 'guosen'
    charset = 'utf8'


class HomeConfig(object):
    host = '192.168.50.232'
    port = 3306
    user = 'root'
    passwd = 'root'
    charset = 'utf8'